package com.link.core.basic.init;

import com.link.core.util.CoreYmlConstants;
import com.link.core.util.RedisUtil;
import com.link.core.util.SpringUtil;
import com.link.core.util.StringUtils;
import com.link.core.util.redisclient.LinkRedisClient;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.context.annotation.DependsOn;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

@Component
@DependsOn({"coreYmlProperties"})
public class IdInit implements ApplicationRunner {
   private static final Logger logger = LoggerFactory.getLogger(IdInit.class);
   private static boolean curExecFlag = false;

   public void run(ApplicationArguments args) throws Exception {
      setIdNum();
      curExecFlag = true;
   }

   public static boolean idInitExecFlag() {
      return curExecFlag;
   }

   public static void setIdNum() throws Exception {
      LinkRedisClient linkRedisClient = null;
      RLock rlock = null;
      long machineIdMin = getMachineId("Min");

      try {
         linkRedisClient = RedisUtil.getJedis();
         if (!linkRedisClient.exists("ALL_GET_ID_NUM")) {
            try {
               rlock = ((RedissonClient)SpringUtil.getBean(RedissonClient.class)).getLock("RLOCK_ID_GET");
               boolean lockFlag = rlock.tryLock(120L, 240L, TimeUnit.SECONDS);
               if (!lockFlag) {
                  throw new Exception("当前初始化程序获取锁失败，请稍后再试：ALL_GET_ID_NUM");
               }

               if (!linkRedisClient.exists("ALL_GET_ID_NUM")) {
                  if (0L == machineIdMin) {
                     linkRedisClient.hset("ALL_GET_ID_NUM", getProperties(), machineIdMin + "_1");
                  } else {
                     linkRedisClient.hset("ALL_GET_ID_NUM", getProperties(), machineIdMin + "_0");
                  }

                  setIdDatacenterIdAndMarId(machineIdMin, 0L);
               } else {
                  whenRediskeyExisist();
               }
            } finally {
               if (null != rlock && rlock.isHeldByCurrentThread()) {
                  rlock.unlock();
               }

            }
         } else {
            whenRediskeyExisist();
         }
      } catch (Exception var14) {
         Exception e = var14;
         logger.error(e.getMessage());
         throw e;
      } finally {
         RedisUtil.returnResource(linkRedisClient);
      }

   }

   private static void whenRediskeyExisist() throws Exception {
      LinkRedisClient linkRedisClient = null;

      try {
         linkRedisClient = RedisUtil.getJedis();
         RLock rlock = null;
         long machineIdMin = getMachineId("Min");
         long machineIdMax = getMachineId("Max");
         Map<String, String> allIdNum = linkRedisClient.hgetAll("ALL_GET_ID_NUM");
         Iterator var11 = allIdNum.keySet().iterator();

         while(var11.hasNext()) {
            String idNum = (String)var11.next();
            if (idNum.equalsIgnoreCase(getProperties())) {
               String value = (String)allIdNum.get(idNum);
               String[] splitNum = value.split("_");
               long machineId = Long.parseLong(splitNum[0]);
               long dataCenterId = Long.parseLong(splitNum[1]);
               setIdDatacenterIdAndMarId(machineId, dataCenterId);
               return;
            }
         }

         try {
            rlock = ((RedissonClient)SpringUtil.getBean(RedissonClient.class)).getLock("RLOCK_ID_GET");
            boolean lockFlag = rlock.tryLock(120L, 240L, TimeUnit.SECONDS);
            if (!lockFlag) {
               throw new Exception("当前初始化程序抢占id获取锁失败，请稍后再试：ALL_GET_ID_NUM");
            }

            for(Collection<String> values = allIdNum.values(); machineIdMin <= machineIdMax; ++machineIdMin) {
               for(long dataCentermin = (long)(machineIdMin == 0L ? 1 : 0); dataCentermin <= 31L; ++dataCentermin) {
                  String value = machineIdMin + "_" + dataCentermin;
                  if (!values.contains(value)) {
                     linkRedisClient.hset("ALL_GET_ID_NUM", getProperties(), value);
                     setIdDatacenterIdAndMarId(machineIdMin, dataCentermin);
                     return;
                  }
               }
            }

            throw new Exception("没有获取到可用的id组合");
         } finally {
            if (null != rlock && rlock.isHeldByCurrentThread()) {
               rlock.unlock();
            }

         }
      } catch (Exception var25) {
         Exception e = var25;
         e.printStackTrace();
         logger.error("获取ID组合报错：" + e.getMessage());
      } finally {
         RedisUtil.returnResource(linkRedisClient);
      }

   }

   private static String getProperties() {
      String ip = ((Environment)SpringUtil.getBean(Environment.class)).getProperty("spring.cloud.client.ip-address");
      String serverName = ((Environment)SpringUtil.getBean(Environment.class)).getProperty("spring.application.name");
      String port = ((Environment)SpringUtil.getBean(Environment.class)).getProperty("server.port");
      return ip.concat(":").concat(serverName).concat(":").concat(port);
   }

   private static void setIdDatacenterIdAndMarId(Long machineId, Long dataCenterId) {
      logger.info("服务：" + getProperties() + "获取到ID组合为：" + machineId + ":" + dataCenterId);
      CoreYmlConstants.setDataCenterId(dataCenterId);
      CoreYmlConstants.setMachineId(machineId);
   }

   private static long getMachineId(String type) {
      String machineIdStr = CoreYmlConstants.getMachineidRange();
      logger.info("获取ID参数，机器ID范围为：" + machineIdStr);
      long machineId;
      if (StringUtils.isBlank(machineIdStr)) {
         if ("Min".equals(type)) {
            machineId = 0L;
         } else {
            machineId = 31L;
         }
      } else {
         String[] idRangeArr = machineIdStr.split(",");
         if ("Min".equals(type)) {
            machineId = Long.parseLong(idRangeArr[0]);
         } else {
            machineId = Long.parseLong(idRangeArr[1]);
         }
      }

      return machineId;
   }
}
